#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl -w

use HTML::Template;
use ActiveState::Config qw( :path);
use PureMessage::Manager::Request qw($REQUEST %in);
use PureMessage::Manager::Config qw(%config);
use PureMessage::Manager::Lang;
use PureMessage::Manager::Response::Services::Postfix;
use PureMessage::MTA::Postfix::Config qw(config_avail get_parms save_parms get_filter save_filter);
use strict;

my %vars;
our %config;

my $r = $REQUEST->get_response( 'PureMessage::Manager::Response::Services::Postfix');

$r->{debug} =0;

$r->add_help(text => "Help", tab => "locserv", page => "mgr_locserv_postfix_contfilt" );

$r->title(text('postfix_filter_title'));

my $action = $in{'action'};

$action = 'save' if $in{'save'};
$action = 'edit' if $in{'edit'};

# revisit is always set when the form is submitted so this is the first view
# so if load isn't sent from the link to this page we should still have
# valid values
#
$in{'load'} = 1 if !$in{revisit};

# get template vars we always want
$vars{'ACTION'} = $action;

my %postfix_parms = get_parms();
my %postfix_filter = get_filter();
my ($service, $host, $port) = split(':', $postfix_parms{'content_filter'});
my ($forward_host, $forward_port) = split(':', $postfix_filter{'forward_port'});

# service should be "pmx" if unset
$service ||= 'pmx';

if ($action =~ /save/) {
    if (validate() ) {
	save_filt();
    } 
} 

if (config_avail()) {
    &display();
} else {
    $r->raise_error("Unable to access Postfix configuration.");
    $r->send();
}

### start of subs

sub display() {
    $vars{HEADER} = $r->ThemeSectionHeader( title=> text('postfix_filter_header'))->output(); 

    $vars{host} = $in{load} ? $host : $in{host};
    $vars{port} = $in{load} ? $port : $in{port};
    $vars{forward_host} = $in{load} ? $forward_host : $in{forward_host};
    $vars{forward_port} = $in{load} ? $forward_port : $in{forward_port};
    $vars{concurrency} = $in{load} ? $postfix_filter{'concurrency'} : $in{concurrency};

    my $tb = $r->ToolBar(
                        buttons => [[ { type=>'image',text=>'Save',name=>'save',icon=>'save'},
                                      { type=>'image',text=>'Cancel',href=>'index.cgi',icon=>'cancel'},
                                   ]],
                        table=> [[1,3],['50%','50%']],
                      );
    $vars{FOOTER} = $r->ThemeSectionFooter(toolbar=>$tb)->output();

    $r->add_template( 'file'=>'tmpl/content_filter.html', 'data'=>\%vars );
    $r->send();
}

sub save_filt {
    my $host = $in{'host'} ? $in{'host'} : '';
    my $port = $in{'port'} ? $in{'port'} : '';
    my $forward_host = $in{'forward_host'} ? $in{'forward_host'} : '';
    my $forward_port = $in{'forward_port'} ? $in{'forward_port'} : '';
    my $concurrency = $in{'concurrency'} ? $in{'concurrency'} : '';

    my $content_filter = $service . ":" . $host . ":" . $port;
    my $forwardport = $forward_host . ":" . $forward_port;
    my %parms = ('content_filter' => $content_filter);
    my %filter = ('concurrency' => $concurrency,
		  'forward_port' => $forwardport);

    if (! save_parms(%parms)) {
        $r->raise_error('postfix_error_saving_filter',$@);
        $r->error_dlg();
        $r->send();
        exit;
    }
    if (! save_filter(%filter)) {
        $r->raise_error('postfix_error_saving_filter',$@);
        $r->error_dlg();
        $r->send();
        exit;
    }
    $r->addInfoBox( msg => text('postfix_saving_ok'));
    $r->log('Update Postfix','','postfix', \%in);
}

sub validate {
    my $in = shift;
    $in{host} = '' unless $in{host};
    $in{port} = '' unless $in{port};
    $in{forward_host} = '' unless $in{forward_host};
    $in{forward_port} = '' unless $in{forward_port};
    $in{concurrency} = '' unless $in{concurrency};
    $REQUEST->typedecl("host", "#str", "Host");
    $REQUEST->typedecl("port", "#int", "Port");
    $REQUEST->typedecl("forward_host", "#str", "Forward Host");
    $REQUEST->typedecl("forward_port", "#int", "Forward Port");
    $REQUEST->typedecl("concurrency", "#int", "Concurrency");
    my @errors = $REQUEST->validate($in);
    if (@errors) {
	$r->raise_error(join("<br>", @errors));
	return;
    } else {
	return 1;
    }
}
