#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl -w

use strict;
use HTML::Template;
use PureMessage::Manager::Request qw($REQUEST %in %text);
use PureMessage::Manager::Config qw(%config);
use PureMessage::Manager::Lang;
use PureMessage::Service;
use PureMessage::Manager::Response::Services::Postfix;
use PureMessage::Manager::Widget;
use PureMessage::Manager::Widget::Form;

my %vars;
our %text;

my $WIDGET = new PureMessage::Manager::Widget;


my $r = $REQUEST->get_response( 'PureMessage::Manager::Response::Services::Postfix',
                                'stderr_to_stdout' => 0
				);

$r->add_help(text => "Help", tab => "locserv", page => "mgr_locserv_postfix" );

$r->title(text('index_title'));

my @services = PureMessage::Service->Available;
my @postfix = grep { $_->type eq 'smtp' && $_->name eq 'Postfix' } @services;
my %postfix_widgets;

updateStatus();

foreach my $postfix (keys %postfix_widgets) {
    my $postfix_w = $postfix_widgets{$postfix}->{w};
    if ($postfix_w->read_input($REQUEST)) {
	my $data = $postfix_w->result_data();
	
	if($data->{start} or $data->{stop} or $data->{restart}) {
	    my $s = $postfix_widgets{$postfix}->{s};
	    my @pid;
	    if ($s and ref($s) and $s->can('start') or $s->can('stop') or $s->can('restart')) {
                if ($data->{start}) {
		    push(@pid,$s->start( bg=>1)) if $data->{start};
                } elsif($data->{stop}) {
		    push(@pid,$s->stop( bg=>1)) if $data->{stop};
                } else {
		    push(@pid,$s->restart( bg=>1)) if $data->{restart};
                }
	    }

	    if (@pid) {
		foreach (@pid) {
		    waitpid($_,0);
		    sleep(1); # postfix takes a while to actually start
		    updateStatus();
		}
	    }
	    
	} 
    }
}

foreach my $postfix (keys %postfix_widgets) {
    $r->add_part($postfix_widgets{$postfix}->{w},$postfix . '_part');
}

$r->send();

# functions

sub updateStatus {
    undef(%postfix_widgets);
    foreach my $postfix (@postfix) {
	my $name = $postfix->name();
	my $raw_status = $postfix->status( );
	my $status;
	my $flds;
	if (! defined $raw_status->{running} ) {
	    $status = {
		'Status' => "Unknown",
	    }; 
	    $flds = ['Status'];
	}
	elsif ($raw_status->{running}) {
	    $status = {
		'Status' => &text('postfix_running_desc','running'),
                'Process Id' => $raw_status->{pid},
                'Ready' => $raw_status->{ready} ? 'Yes' : 'No',
                'VM Size' => $raw_status->{vsz},
	    }; 
	    $flds = ['Status', 'Process Id', 'Ready', 'VM Size'];
	} else {
	    $status = { 
		'Status' => &text('postfix_running_desc','stopped'),
	    };
	    $flds = ['Status'];
	}

	my @buttons;
	push(@buttons, {type=>'image',icon=>'start', name=>'start',text=>'Start'}) 
	    if defined $postfix->running and ! $postfix->running;
	push(@buttons, {type=>'image',icon=>'restart',name=>'restart',text=>'Restart'},
	     {type=>'image',icon=>'stop',name=>'stop',text=>'Stop'} ) 
	    if $postfix->running;
	
	my $tb = $WIDGET->widget( CLASS => 'PureMessage::Manager::Widget::ToolBar',
				  buttons => [ \@buttons, ], );
	
	my $new_widget = $r->Form(
				  name => "postfix_$name",
				  data => $status,
				  columns=>2,
				  fields => $flds,
				  title=>&text('postfix_status_dialog_title'),
				  toolbar => $tb,
				  action => 'index.cgi',
				  keys => sub { return ucfirst(shift)},
				  padding => '<BR> <BR> ',
				  );
	
	$postfix_widgets{$name} = { w=>$new_widget,s=>$postfix};
    }
}

1;

