#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl -w

eval 'exec /tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl -w -S $0 ${1+"$@"}'
    if 0; # not running under some shell
use strict;
use warnings;
use PureMessage::Version qw($PREFIX);
use ActiveState::Handy qw(cat);

my $cmd = shift;
print usage() unless $cmd;

my %commands = ('start'  => \&pf_start,
		'stop'   => \&pf_stop,
		'restart'=> \&pf_restart,
		'reload' => \&pf_reload,
		'status' => \&pf_status,
		'conf'   => \&pf_conf,
		'filterconf'   => \&pf_fconf,
                'mtime'  => \&pf_mtime
	       );

my @opts = @ARGV;

my $pf_cmd = "$PREFIX/postfix/sbin/postfix";
my @pf_cmd_args = ("-c", "$PREFIX/postfix/etc");
my $pf_conf_cmd = "$PREFIX/postfix/sbin/postconf";
my $pf_map_cmd   = "$PREFIX/postfix/sbin/postmap";
my $pf_alias_cmd = "$PREFIX/postfix/bin/newaliases";

my $pf_queue_dir = qx{$pf_conf_cmd @pf_cmd_args -h queue_directory 2>/dev/null} || '';
chomp $pf_queue_dir;
my $pf_pid_file = "$pf_queue_dir/pid/master.pid";

# Check that we are running as the correct user
my $pfuser = (stat("$PREFIX/postfix"))[4];
if ($> != $pfuser) {
    my $cur_id = getpwuid $>;
    my $need_id = getpwuid $pfuser;
    die "This program must be run as user '$need_id', not as '$cur_id'.\n";
}

# get and do the action
my $action = $commands{$cmd};
usage() unless $action;
my $fn;

exit &$action(@ARGV);

sub usage{
    (my $prog = $0) =~ s,.*/,,;
    $prog =~ s#\.pl$##;
    print <<USAGE;
Usage: $prog start|stop|restart|status

Any additional parameters are passed to the Postfix control program.
USAGE
    exit;
}

# Determine the Postfix PID
sub pf_pid{
    my $pid = cat($pf_pid_file);
    if ($pid and $pid =~ m/^\s*(\d+)/) {
	$pid =~ $1;
	undef $pid unless kill(0, $pid);
    }
    return $pid; 
}

sub pf_mtime {
    my @stat = stat($pf_pid_file);
    my $rc=1;
    if(@stat) {
        print $stat[9];
        $rc=0;
    }
    return $rc;
}

sub pf_maps {
    my $rc = 0;
    foreach my $map qw{access canonical relocated transport virtual} {
        unless (-f "$PREFIX/postfix/etc/$map.db") {
            $rc = system($pf_map_cmd, "$PREFIX/postfix/etc/$map") >> 8;
           last if $rc;
        }
    }

    unless (-f "$PREFIX/postfix/etc/aliases.db" || $rc) {
        $rc = system($pf_alias_cmd) >> 8;
    }

    return $rc;
}

sub pf_start{
    my $rc = 0;
    if (pf_pid()) {
        print "Postfix is already running.\n";
        return 0;
    }
    else {
        $rc = pf_maps();
        unless ($rc) {
            $rc = system($pf_cmd, @pf_cmd_args, "start", @opts) >> 8;
            sleep(1);
        }
    }
    unless (pf_pid()) {
        print <<EOT;

    Postfix failed to start.  Please check your system logs to determine
    the error.

EOT
    }
    return $rc;
}

sub pf_stop{
    if (pf_pid()) {
        return system($pf_cmd, @pf_cmd_args, "stop", @opts) >> 8;
    }
    else {
        print "Postfix is not running.\n";
        return 2;
    }
}

sub pf_reload {
    if (pf_pid()) {
        return system($pf_cmd, @pf_cmd_args, "reload", @opts) >> 8;
    }
    else {
        return pf_start();
    }
}

sub pf_restart {
    pf_stop() if pf_pid();
    return pf_start();
}

sub pf_status_hash {
    use PureMessage::_sys;
    my %info = ();
    $info{'pid'}      = pf_pid();
    $info{'running'}  = $info{pid};
    return \%info;
}

sub pf_status {
    my $info = pf_status_hash();
    unless ($$info{'running'}) {
	print "Postfix is not running.\n";
	return 2;
    }
    print "Postfix is running:\n";
    print "   pid:     $$info{'pid'}\n";
    return 0;
}

sub pf_conf {
    return system($pf_conf_cmd, @pf_cmd_args, @opts) >> 8;
}

sub pf_fconf {
    my $opt = shift @opts;
    my $mastercf = "$PREFIX/postfix/etc/master.cf";
    if ($opt eq 'get') {
	my $serv = "";
	my $fport = "";
	my $ccur = "";

	open my $mcf, $mastercf or die "Can't open $mastercf: $!";
	while (<$mcf>) {
	    if (/^# BEGIN pmx-send FILTER CONFIG/) {
		while (<$mcf>) {
		    if (/^(S+)\s+unix\s+-\s+-\s+n\s+-\s+(\d+)\s+smtp\b/) {
			$serv = $1;
			$ccur = $2 if !$ccur || $ccur < $2;
		    }
		    last if /^# END pmx-send FILTER CONFIG/;
		}
		next;
	    }
	    elsif (/^# BEGIN pmx-recv FILTER CONFIG/) {
		while (<$mcf>) {
		    if (/^(\S+)\s+inet\s+n\s+-\s+n\s+-\s+(\d+)\s+smtpd\b/) {
			$fport = $1;
			$ccur = $2 if !$ccur || $ccur < $2;
		    }
		    last if /^# END pmx-recv FILTER CONFIG/;
		}
		next;
	    }
	}
	close $mcf;
	print <<EOT;
service=$serv
forward_port=$fport
concurrency=$ccur
EOT
    }
    elsif ($opt eq 'set') {
	my %opts = map { split("=", $_, 2) } @opts;
	my $serv = $opts{service} || "pmx";
	my $fport = $opts{forward_port} || "localhost:10026";
	my $ccur = $opts{concurrency} || "10";

	my $found_smtp = 0;
	my $found_smtpd = 0;
	open my $mcf, $mastercf or die "Can't open $mastercf: $!";
	open my $nmcf, ">$mastercf.tmp" or die "Can't open $mastercf.tmp: $!";
	while (<$mcf>) {
	    if (/^# BEGIN pmx-send FILTER CONFIG/) {
		print $nmcf $_;
		print $nmcf "$serv\tunix\t-\t-\tn\t-\t$ccur\tsmtp\n";
		$_ = <$mcf>;
		while (<$mcf>) {
		    print $nmcf $_;
		    last if /^# END pmx-send FILTER CONFIG/;
		}
		++$found_smtp;
		next;
	    }
	    elsif (/^# BEGIN pmx-recv FILTER CONFIG/) {
		print $nmcf $_;
		print $nmcf "$fport\tinet\tn\t-\tn\t-\t$ccur\tsmtpd\n";
		$_ = <$mcf>;
		while (<$mcf>) {
		    print $nmcf $_;
		    last if /^# END pmx-recv FILTER CONFIG/;
		}
		++$found_smtpd;
		next;
	    }
	    print $nmcf $_;
	}
	close $mcf;
	unless ($found_smtp) {
	    print $nmcf <<EOT;
# BEGIN pmx-send FILTER CONFIG # DO NOT EDIT THIS LINE!
$serv\tunix\t-\t-\tn\t-\t$ccur\tsmtp
# END pmx-send FILTER CONFIG # DO NOT EDIT THIS LINE!

EOT
	}
	unless ($found_smtpd) {
	    print $nmcf <<EOT;
# BEGIN pmx-recv FILTER CONFIG # DO NOT EDIT THIS LINE!
$fport\tinet\tn\t-\tn\t-\t$ccur\tsmtpd
    -o content_filter=
    -o local_recipient_maps=
    -o relay_recipient_maps=
    -o myhostname=localhost
    -o smtpd_helo_restrictions=
    -o smtpd_client_restrictions=
    -o smtpd_sender_restrictions=
    -o smtpd_recipient_restrictions=permit_mynetworks,reject
    -o mynetworks=127.0.0.0/8
    -o allow_untrusted_routing=yes
# END pmx-recv FILTER CONFIG # DO NOT EDIT THIS LINE!

EOT
	}
	close $nmcf or die "Can't flush $mastercf.tmp: $!";
	system("cp -pf $mastercf $mastercf.bak");
	rename("$mastercf.tmp", $mastercf);
    }
    else {
        warn "Unknown 'pmx-mta-postfix filterconf' option '$opt'";
	return 1;
    }
    return 0;
}

__END__

=head1 NAME

pmx-mta-postfix - Postfix control application

=head1 SYNOPSIS

    pmx-mta-postfix start
    pmx-mta-postfix stop
    pmx-mta-postfix restart
    pmx-mta-postfix reload
    pmx-mta-postfix status
    pmx-mta-postfix conf [args]

=head1 DESCRIPTION

C<pmx-mta-postfix> is a control program for the PureMessage installation of
the Postfix MTA.  It is used to perform the regular operations
on the Postfix server.  This program is to be run by the owner of the 
<pmx-prefix>/postfix directory.

The following sub-commands are recognized:

=over

=item B<pmx-mta-postfix start>

Start Postfix if it is not already running.

=item B<pmx-mta-postfix stop>

Stop Postfix if it is running.

=item B<pmx-mta-postfix restart>

Stop and start Postfix.

=item B<pmx-mta-postfix reload>

Does a soft restart of Postfix.

=item B<pmx-mta-postfix status>

Print status information about Postfix.

=item B<pmx-mta-postfix conf> [args]

Runs C<postconf>, passing it any arguments that were provided.

=back

=head1 COPYRIGHT

Copyright (c) 2003 ActiveState.  All rights reserved.

=cut
