#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl -w

use HTML::Template;
use ActiveState::Config qw( :path);
use PureMessage::Manager::Request qw($REQUEST %in);
use PureMessage::Manager::Config qw(%config);
use PureMessage::Manager::Lang;
use PureMessage::Manager::Response::Services::Postfix;
use PureMessage::MTA::Postfix::Config qw(config_avail get_parms save_parms);
use strict;

my %vars;
our %config;

my $r = $REQUEST->get_response( 'PureMessage::Manager::Response::Services::Postfix');

$r->{debug} =0;

$r->add_help(text => "Help", tab => "locserv", page => "mgr_locserv_postfix_configure" );

$r->title(text('postfix_configure_title'));

my $action = $in{'action'};

$action = 'save' if $in{'save'};
$action = 'edit' if $in{'edit'};

# revisit is always set when the form is submitted so this is the first view
# so if load isn't sent from the link to this page we should still have
# valid values
#
$in{'load'} = 1 if !$in{revisit};

# get template vars we always want
$vars{'ACTION'} = $action;

if ($action =~ /save/) {
    if (validate(\%in)) {
	save_config();
    } 
} 

my %postfix_config = get_parms();

if (config_avail()) {
    &display();
} else {
    $r->raise_error("Unable to access Postfix configuration.");
    $r->send();
}

### start of subs

sub display() {
    $vars{HEADER} = $r->ThemeSectionHeader( title=> text('postfix_edit_header'))->output(); 

    $vars{name} = $in{load} ? $postfix_config{'myhostname'} : $in{name};
    $vars{domain} = $in{load} ? $postfix_config{'mydomain'} : $in{domain};
    $vars{originating_address} = $in{load} ? $postfix_config{'myorigin'} : $in{originating_address};
    $vars{destination_address} = $in{load} ? $postfix_config{'mydestination'} : $in{destination_address};
    $vars{mail_relay} = $in{load} ? $postfix_config{'mynetworks'} : $in{mail_relay};
    $vars{gateway} = $in{load} ? $postfix_config{'relayhost'} : $in{gateway};

    my $tb = $r->ToolBar(
                        buttons => [[ { type=>'image',text=>'Save',name=>'save',icon=>'save'},
                                      { type=>'image',text=>'Cancel',href=>'index.cgi',icon=>'cancel'},
                                   ]],
                        table=> [[1,3],['50%','50%']],
                      );
    $vars{FOOTER} = $r->ThemeSectionFooter(toolbar=>$tb)->output();

    $r->add_template( 'file'=>'tmpl/postfix.html', 'data'=>\%vars );
    $r->send();
}

sub save_config {
    my %values = ( 'myhostname' => $in{name} ? $in{name} : '',
		   'mydomain' => $in{domain} ? $in{domain} : '',
		   'myorigin' => $in{originating_address} ? $in{originating_address} : '',
		   'mydestination' => $in{destination_address} ? $in{destination_address} : '',
		   'mynetworks' => $in{mail_relay} ? $in{mail_relay} : '',
		   'relayhost' => $in{gateway} ? $in{gateway} : '',
		   );
    if (! save_parms(%values)) {
        $r->raise_error('postfix_error_saving_conf',$@);
        $r->error_dlg();
        $r->send();
        exit;
    }
    $r->addInfoBox( msg => text('postfix_saving_ok'));
    $r->log('Update Postfix','','postfix', \%in);
}

sub validate {
    my $in = shift;
    $in{name} = '' unless $in{name};
    $in{domain} = '' unless $in{domain};
    $in{originating_address} = '' unless $in{originating_address};
    $in{destination_address} = '' unless $in{destination_address};
    $in{mail_relay} = '' unless $in{mail_relay};
    $in{gateway} = '' unless $in{gateway};
    $REQUEST->typedecl("name", "#str", "Name of Mail Server");
    $REQUEST->typedecl("domain", "#str", "Domain");
    $REQUEST->typedecl("originating_address", "#str", "Originating Address");
    $REQUEST->typedecl("destination_address", "#str", "Destination Address");
    $REQUEST->typedecl("mail_relay", "#str", "Mail Relay");
    $REQUEST->typedecl("gateway", "#str", "Gateway");
    my @errors = $REQUEST->validate($in);
    if (@errors) {
	$r->raise_error(join("<br>", @errors));
	return;
    } else {
	return 1;
    }
}
