package PureMessage::MTA::Postfix::Service;
use strict;
use warnings;
use Carp;
use PureMessage::Version qw($PREFIX);
use PureMessage::Service::Config qw(is_stale);

use base 'Exporter';
our @EXPORT = qw(
    mta_start
    mta_stop
    mta_restart
    mta_reload
    mta_status
);

my $pfctl = "$PREFIX/postfix/sbin/pmx-mta-postfix";

sub mta_start {
    system($pfctl, "start", @_) >> 8;
}

sub mta_stop {
    system($pfctl, "stop", @_) >> 8;
}

sub mta_restart {
    system($pfctl, "restart", @_) >> 8;
}

sub mta_status {
    my %info;
    my $s;
    my $mtime;
    if (-x $pfctl) {
        $s = `$pfctl status @_`;    # XXX sucks to have to parse this
        $mtime = `$pfctl mtime`;
    } else {
	return { error => 'Postfix is not installed.' };
    }
    if (defined $s and $s =~ /\s+pid:\s+(\d+)\n/) {
        $info{pid} = $1;
        $info{running} = $1 ? 1 : 0;
        $info{ready} = 1;       # XXX - ping?
    } elsif (defined $s and $s =~ /Postfix is not running\./) {
	return { running => 0 }
    } else {
	my $err = $? ? ($? >> 8) : 1;
        # The 'running' key is left out to indicate to the manager that
        # the status is Unknown.
	return { error => $err };
    }

    # XXX stolen from PureMessage::Service::Sendmail
    my $ps = "UNIX95=1 /bin/ps";
    my $ps_opts;
    my $ps_cmd;
    if ($^O eq "freebsd") {
        $ps_opts = "axc";
        $ps_cmd = "command";
    }
    elsif($^O eq "hpux" || $^O eq "linux") {
        $ps_opts = "e";
        $ps_cmd = "comm";
    }
    elsif($^O eq "solaris"){
        $ps_opts = "eL";
        $ps_cmd = "comm";
    }
    elsif($^O eq "aix"){
        $ps_opts = "e";
        $ps_cmd = "comm";
    }

    my $cli = "$ps -${ps_opts}o pgid,pid,ppid,vsz,$ps_cmd";
    if (open(PS, "$cli | ")) {
        while (<PS>) {
            my($pgid, $pid, $ppid, $vsz, $cmd) = split(' ', $_);
	    # skip the first list
	    next unless $pid =~ m/^\d+\z/;
            next unless ($pid == $info{pid});
            $info{vsz} += $vsz;
        }
    }

    my $is_stale_time = is_stale('smtp') || 0;

    if ($is_stale_time > $mtime) {
        $info{'config'} = 'stale';
    }
    else {
        $info{'config'} = 'current';
    }
    
    return \%info;    
}

sub mta_reload {
    system($pfctl, "reload", @_) >> 8;
}

1;
