package PureMessage::Service::postfix;
use strict;
use base 'PureMessage::Service';
use PureMessage::MTA::Postfix::Service qw(mta_start mta_stop
                                          mta_restart mta_reload
					  mta_status);
sub Available {
    my $class = shift;
    my @avail;
    push(@avail, $class->new(name => "Postfix"));
    return @avail;
}

sub _bg {
    my $pid = fork();
    die "Can't fork: $!" unless defined $pid;
    return $pid if $pid;

    open(STDIN, "</dev/null");
    open(STDOUT, ">/dev/null");
    open(STDERR, ">/dev/null");
    return 0;
}

sub start {
    my($self, %opts) = @_;
    if ($opts{bg}) {
        if (my $pid = _bg()) {
           return $pid;
        }
    }
    mta_start();
    exit if $opts{bg};
}

sub stop {
    my($self, %opts) = @_;
    if ($opts{bg}) {
        if (my $pid = _bg()) {
            return $pid;
        }
    }
    mta_stop();
    exit if $opts{bg};
}

sub reload {
    my($self, %opts) = @_;
    if ($opts{bg}) {
        if (my $pid = _bg()) {
            return $pid;
        }
    }
    mta_reload();
    exit if $opts{bg};
}

sub restart {
    my($self, %opts) = @_;
    if ($opts{bg}) {
        if (my $pid = _bg()) {
            return $pid;
        }
    }
    mta_restart();
    exit if $opts{bg};
}

sub status {
    my $self = shift;

    my $info = mta_status();
    $info->{ready}   = ($info->{pid} ? 1 : 0);
    return($info);
}

sub running {
    my $self = shift;
    return mta_status()->{running};
}

# overload type() method in PureMessage::Service
sub type {
    return 'smtp';
}

1;
